<?php

namespace Drupal\fieldable_path\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'fieldable_path' field type.
 *
 * @FieldType(
 *   id = "fieldable_path",
 *   label = @Translation("Fieldable Path"),
 *   module = "fieldable_path",
 *   description = @Translation("Field for mirroring of 'path' property in entities."),
 *   default_widget = "fieldable_path_widget",
 *   default_formatter = "fieldable_path_formatter"
 * )
 */
class FieldablePath extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE,
          'default' => '',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    // We need to say that our field is not empty
    // in order to get preSave() method executed.
    // TODO: Any drawbacks of doing this?
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('Entity path'));
    return $properties;
  }

}
